using System;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Timers;
using System.Collections;
using System.Collections.Generic;

namespace ProjectCarbon
{
    class PacketCapturedEventArgs : EventArgs
    {
        private byte[] m_packetBuffer;

        public byte[] Buffer
        {
            get { return m_packetBuffer; }
            set { m_packetBuffer = value; }
        }
	
    }

    class PacketCapture: IDisposable
    {
        public delegate void PacketCapturedEventHandler(object sender, 
            PacketCapturedEventArgs e);

        public event PacketCapturedEventHandler OnPacketCaptured;
        private Socket monitorSocket;
        private IPAddress ipAddress;
        private byte[] packetBuffer = new byte[65535];
        private Timer timer;
        //private ArrayList adapters;
        private List<NetworkAdapter> adapters;
        private List<NetworkAdapter> monitoredAdapters;
        private IPAddress[] localIpAddresses = 
            Dns.GetHostEntry(Dns.GetHostName()).AddressList;

        private void EnumerateNetworkAdapters()
        {
            PerformanceCounterCategory category = new PerformanceCounterCategory("Network Interface");
            foreach (string name in category.GetInstanceNames())
            {
                if (name == "MS TCP Loopback interface")
                    continue;
                NetworkAdapter adapter = new NetworkAdapter(name);
                adapter.dlCounter = new PerformanceCounter("Network Interface", 
                    "Bytes Received/sec", name);
                adapter.ulCounter = new PerformanceCounter("Network Interface", 
                    "Bytes Sent/sec", name);
                this.adapters.Add(adapter);			// Add it to ArrayList adapter
            }
        }

        public PacketCapture (IPAddress ip)
	    {
            if (Environment.OSVersion.Platform != PlatformID.Win32NT ||
                Environment.OSVersion.Version.Major < 5)
            {
                throw new NotSupportedException("This library requires Windows 2000, Windows XP or Windows .NET Server!");
            }
            ipAddress = ip;
            this.adapters = new List<NetworkAdapter>();
            this.monitoredAdapters = new List<NetworkAdapter>();
            EnumerateNetworkAdapters();
            timer = new Timer(1000);
            timer.Elapsed += new ElapsedEventHandler(timer_Elapsed);
	    }

        private void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            lock (monitoredAdapters)
            {
                if (this.monitoredAdapters.Count == 0)
                    return;

                foreach (NetworkAdapter adapter in this.monitoredAdapters)
                    adapter.refresh();
            }
        }

        public NetworkAdapter[] Adapters
        {
            get
            {
                return (NetworkAdapter[])this.adapters.ToArray();
            }
        }
        public IPAddress[] IPAddresses
        {
            get
            {
                return localIpAddresses;
            }
        }

        public void StartCapture()
        {
            try
            {
                monitorSocket = new Socket(AddressFamily.InterNetwork,
                    SocketType.Raw, ProtocolType.IP);

                monitorSocket.Bind(new IPEndPoint(ipAddress, 0));

                monitorSocket.IOControl(IOControlCode.ReceiveAll,
                    BitConverter.GetBytes(1), null);

                monitorSocket.BeginReceive(
                    packetBuffer, 0, packetBuffer.Length,
                    SocketFlags.None, new AsyncCallback(OnReceive),
                    null);
            }
            catch
            {
                monitorSocket = null;
                throw new SocketException();
            }
        }

        public void StopCapture()
        {
            if (monitorSocket != null)
                monitorSocket.Close();
            //monitorSocket = null;
  
        }
        private void OnReceive(IAsyncResult ar)
        {
            try
            {
                int recvd = monitorSocket.EndReceive(ar);
                byte[] recvdBuffer = new byte[recvd];
                Array.Copy(packetBuffer, 0, recvdBuffer, 0, recvd);

                PacketCapturedEventArgs e = new PacketCapturedEventArgs();
                e.Buffer = recvdBuffer;
                OnPacketCaptured(this, e);

                monitorSocket.BeginReceive(
                    packetBuffer, 0, packetBuffer.Length,
                    SocketFlags.None, new AsyncCallback(OnReceive), null);
            }
            catch (ObjectDisposedException)
            {
                Console.WriteLine("OnReceive: Socket has been closed.");
            }
        }

        /// <summary>
        /// Enable the timer and add all adapters to the monitored adapters list
        /// unless the list is empty.
        /// </summary>
        public void StartMonitoring()
        {
            if (this.adapters.Count > 0)
            {
                foreach (NetworkAdapter adapter in this.adapters)
                    if (!this.monitoredAdapters.Contains(adapter))
                    {
                        this.monitoredAdapters.Add(adapter);
                        adapter.init();
                    }

                timer.Enabled = true;
            }
        }

        // Enable the timer, and add the specified adapter to the monitoredAdapters list
        public void StartMonitoring(NetworkAdapter adapter)
        {
            if (!this.monitoredAdapters.Contains(adapter))
            {
                this.monitoredAdapters.Add(adapter);
                adapter.init();
            }
            timer.Enabled = true;
        }

        // Disable the timer, and clear the monitoredAdapters list.
        public void StopMonitoring()
        {
            this.monitoredAdapters.Clear();
            timer.Enabled = false;
        }

        // Remove the specified adapter from the monitoredAdapters list, and disable the timer if the monitoredAdapters list is empty.
        public void StopMonitoring(NetworkAdapter adapter)
        {
            if (this.monitoredAdapters.Contains(adapter))
                this.monitoredAdapters.Remove(adapter);
            if (this.monitoredAdapters.Count == 0)
                timer.Enabled = false;
        }

        #region IDisposable Members

        public void Dispose()
        {
            timer.Dispose();
            monitorSocket.Close();
        }

        #endregion
    }
}